"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.downloadAndExtract = exports.getChecksum = exports.verifyChecksum = exports.download = exports.getDownloadedFileInfo = void 0;
const tslib_1 = require("tslib");
const crypto = tslib_1.__importStar(require("crypto"));
const axios = tslib_1.__importStar(require("axios"));
const fs = tslib_1.__importStar(require("fs-extra"));
const path = tslib_1.__importStar(require("path"));
const fs_1 = require("./fs");
const getDownloadedFileInfo = (url, downloadDir) => {
    const pathWithoutProtocol = url.replace(/(^\w+:|^)\/\//, '').split('/');
    const fileName = pathWithoutProtocol.pop();
    const filePath = path.join(downloadDir, fileName);
    return {
        fileName,
        fileDir: downloadDir,
        filePath,
    };
};
exports.getDownloadedFileInfo = getDownloadedFileInfo;
const download = async (url, dir) => {
    const { filePath, fileDir } = (0, exports.getDownloadedFileInfo)(url, dir);
    if (fs.existsSync(filePath)) {
        fs.unlinkSync(filePath);
    }
    fs.ensureDirSync(fileDir);
    const writeStream = fs.createWriteStream(filePath);
    const response = await axios.default({
        url,
        method: 'GET',
        responseType: 'stream',
        maxContentLength: 5000,
    });
    response.data.pipe(writeStream);
    return new Promise((resolve, reject) => {
        writeStream.on('finish', resolve);
        writeStream.on('error', reject);
    });
};
exports.download = download;
const verifyChecksum = async (filePath, expectedChecksum) => {
    const fileStream = fs.createReadStream(filePath);
    const dataHash = crypto.createHash('sha256');
    const fileHash = await new Promise((resolve, reject) => {
        fileStream.on('data', (datum) => {
            dataHash.update(datum);
        });
        fileStream.on('error', error => {
            reject(error);
        });
        fileStream.on('end', () => {
            resolve(dataHash.digest());
        });
    });
    const fileChecksum = fileHash.toString('hex');
    if (fileChecksum !== expectedChecksum) {
        throw new Error(`File checksum: ${fileChecksum} mismatched with expected checksum: ${expectedChecksum}`);
    }
};
exports.verifyChecksum = verifyChecksum;
const getChecksum = (url, dir) => {
    const { filePath } = (0, exports.getDownloadedFileInfo)(url, dir);
    const content = fs.readFileSync(`${filePath}.SHA256`, 'utf8');
    if (!content) {
        throw new Error(`Invalid filepath: ${filePath}`);
    }
    return content.split(' ')[0];
};
exports.getChecksum = getChecksum;
const downloadAndExtract = async (url, downloadPath, extractionPath) => {
    await (0, exports.download)(url, downloadPath);
    await (0, exports.download)(`${url}.SHA256`, downloadPath);
    const { filePath } = (0, exports.getDownloadedFileInfo)(url, downloadPath);
    const checksum = (0, exports.getChecksum)(url, downloadPath);
    await (0, exports.verifyChecksum)(filePath, checksum);
    await (0, fs_1.extractTarBall)(filePath, extractionPath);
};
exports.downloadAndExtract = downloadAndExtract;
//# sourceMappingURL=download.js.map